let tooltip = null;
let jump = null;
const logo = chrome.runtime.getURL('logo.png');

// 判断鼠标选中文本位置，判断 tooltip 展示位置，如果 tooltip 超出屏幕则向左展示
function positionTooltip() {
    if (tooltip) {
        const selection = window.getSelection();
        const height = tooltip.offsetHeight;
        if (!selection.isCollapsed) {
            const rect = selection.getRangeAt(0).getBoundingClientRect();
            let left = rect.left + window.scrollX + rect.width + 10
            if (left > window.innerWidth - 300) {
                left = rect.left - 360
            }
            let top = rect.top - 21
            if (top > window.innerHeight - height) {
                top = rect.top - height + 21
            }
            tooltip.style.top = `${top}px`;
            tooltip.style.left = `${left}px`;
        }
    }
}

function removeTooltip() {
    if (tooltip) {
        document.body.removeChild(tooltip);
        tooltip = null;
    }
}

function createTooltip(data) {
    if (tooltip) removeTooltip()
    if (jump) {
        jump.removeEventListener('click', () => {
            chrome.runtime.sendMessage({ action: "openNewTab", text: data.ip_info.ip });
        })
        jump = null;
    }

    tooltip = document.createElement('div');
    tooltip.style.cssText = `
        position: fixed;
        z-index: 10000;
        background: white;
        box-shadow: 0px 20px 20px 0px rgba(0,0,0,0.2);
        padding: 12px; 
        font-size: 14px;
        border-radius: 4px;
        width: 350px;
    `;

    const info = document.createElement('div');
    info.style.cssText = `
        display: flex;
        flex-direction: column;
        gap: 8px;
        font-size: 14px;
        color: #000;
    `;

    if (data.ip_info.ip) {
        const location = data.ip_info.address.country !== '中国' ? `${data.ip_info.address.country}` : `${data.ip_info.address.province} ${data.ip_info.address.city}`;
        info.innerHTML = `
            <div style='display: flex;justify-content: space-between;align-items: center;gap:16px;'>
                <div style='display: flex;align-items: center;gap: 4px'>
                    <img width='16' height='16' src='${logo}' />
                    长亭 IP 情报分析查询结果
                </div>
                <div id='ip-jump' style='color: #206cff;cursor: pointer;text-align: right;'>查看更多</div>
            </div>
            <div style='display: flex;flex-direction: column;gap: 8px;font-size: 12px;padding: 16px;border: 1px solid #eee;border-radius: 4px;'>
                <div style='display: flex; align-items: flex-start;'><span style='color: rgba(0,0,0,0.3);flex-shrink:0; width: 64px'>地理位置</span>${location.trim() || '-'}</div>
                <div style='display: flex; align-items: flex-start;'><span style='color: rgba(0,0,0,0.3);flex-shrink:0; width: 64px'>ISP</span>${data.ip_info.address.isp || '-'}</div>
                <div style='display: flex; align-items: flex-start;'><span style='color: rgba(0,0,0,0.3);flex-shrink:0; width: 64px'>运营商</span>${data.ip_info.address.owner || '-'}</div>
            </div>
        `;
        if (data.events && data.events.length > 0) {
            const total = document.createElement('div');
            total.style.cssText = `
                display: flex;
                align-items: center;
                gap: 8px;
                padding: 8px 16px;
                background: rgba(255,24,68,0.04);
                border: 1px solid rgba(255,24,68,0.4);
                border-radius: 4px;
                color: #FF1844;
                line-height: 22px;
            `;
            total.innerHTML = `
                <svg t="1728617918391" class="icon" viewBox="0 0 1024 1024" version="1.1" xmlns="http://www.w3.org/2000/svg" p-id="4979" width="14" height="14"><path d="M512 0a512.073143 512.073143 0 0 0 0 1024A512.073143 512.073143 0 0 0 512 0z m296.521143 808.521143A416.694857 416.694857 0 0 1 512 931.401143a416.694857 416.694857 0 0 1-296.521143-122.88A416.694857 416.694857 0 0 1 92.598857 512c0-111.981714 43.666286-217.307429 122.88-296.521143A416.694857 416.694857 0 0 1 512 92.598857c111.981714 0 217.307429 43.666286 296.521143 122.88A416.694857 416.694857 0 0 1 931.401143 512a416.182857 416.182857 0 0 1-122.88 296.521143z" p-id="4980" fill="#FF1844"></path><path d="M590.116571 672.182857v86.308572c0 13.458286-11.849143 24.576-25.965714 24.576H459.995429c-14.262857 0-25.965714-11.190857-25.965715-24.576V672.182857c0-13.458286 11.849143-24.502857 25.965715-24.502857h104.155428a25.307429 25.307429 0 0 1 25.965714 24.576z m12.214858-406.674286L590.994286 561.298286c-0.365714 13.458286-12.580571 24.576-26.843429 24.576H459.995429c-14.262857 0-26.477714-11.264-26.843429-24.576L421.668571 265.508571a24.502857 24.502857 0 0 1 25.234286-24.576h130.194286c14.262857 0 25.746286 11.190857 25.234286 24.576z" p-id="4981" fill="#FF1844"></path></svg>
                <span>此 IP 在 30 天内存在 ${data.events.length} 次攻击行为</span>
            `;

            info.appendChild(total);
        }
    }

    tooltip.appendChild(info);
    document.body.appendChild(tooltip);

    jump = document.getElementById('ip-jump');
    jump.addEventListener('click', () => {
        chrome.runtime.sendMessage({ action: "openNewTab", text: data.ip_info.ip });
    })
}

function createErrorTooltip(msg) {
    if (tooltip) removeTooltip()

    tooltip = document.createElement('div');
    tooltip.style.cssText = `
        position: fixed;
        z-index: 10000;
        background: white;
        border: 1px solid #eee;
        padding: 12px; 
        font-size: 14px;
        border-radius: 4px;
        maxWidth: 250px;
    `;

    const info = document.createElement('div');
    info.style.cssText = `
        color: #FF1844;
        line-height: 22px;
        font-size: 14px;
    `;

    info.innerText = msg.split('[')[0];

    tooltip.appendChild(info);
    document.body.appendChild(tooltip);
}

document.addEventListener('mouseup', (event) => {
    const selection = window.getSelection().toString().trim();
    if (selection && /^(\d{1,3}\.){3}\d{1,3}$/.test(selection)) {
        chrome.storage.local.get('appkey', (result) => {
            if (result.appkey) {
                fetch(`https://ip-0.rivers.chaitin.cn/api/share/s?ip=${selection}&sk=${result.appkey}`).then(res => res.json()).then(res => {
                    if (res.data) createTooltip(res.data)
                    if (res.message) createErrorTooltip(res.message)
                    positionTooltip();
                });
            }
        })
    }
    if (tooltip) {
        const tooltipRect = tooltip.getBoundingClientRect();
        const clickX = event.clientX;
        const clickY = event.clientY;
        const outTooltip = clickX < tooltipRect.left || clickX > tooltipRect.right || clickY < tooltipRect.top || clickY > tooltipRect.bottom
        if (outTooltip) {
            removeTooltip();
        }
    }
});